import { h } from '@stencil/core';
let nextId = 0;
/**
 * An entry to be displayed as part of a `DetailDialog`.
 */
export class DetailDialogItem {
    constructor() {
        this.index = nextId++;
        this.itemId = 0;
        /**
         * A flag indicating that this item will be removed when all the details are
         * serialized and sent to the server.
         */
        this.remove = false;
        /**
         * The description for this item.
         */
        this.description = '';
        /**
         * The name of this item.
         */
        this.name = '';
        /**
         * Alternate text for this item.
         */
        this.altText = '';
        /**
         * The ID of the detail type of this item.
         */
        this.detailTypeId = -1;
        /**
         * The short code of this item.
         */
        this.code = '';
        /**
         * The currently selected category.
         */
        this.categorySelection = 0;
        /**
         * All the possible values for the type of this `DetailDialogItem`.
         */
        this.typeOptions = [];
        /**
         * The currently selected item Type.
         */
        this.typeSelection = 0;
    }
    onDetailChanged() {
        if (this.detail) {
            this.description = this.detail.description;
            this.name = this.detail.name;
            this.altText = this.detail.altText;
            this.itemId = this.detail.id;
            this.detailTypeId = this.detail.detailTypeId;
            this.code = this.detail.code;
            if (this.detail.detailTypeId > 0) {
                // Set the selected category and detailType to match.
                this.categoryOptions.forEach((c, cidx) => {
                    c.items.forEach((dt, dtidx) => {
                        if (this.detailTypeId === dt.id) {
                            this.updateCategory(cidx);
                            this.typeSelection = dtidx; // { ...dt };
                        }
                    });
                });
            }
        }
    }
    onCategoryOptionsChanged() {
        this.updateCategory(0);
    }
    updateCategory(newIndex) {
        this.categorySelection = newIndex;
        this.typeOptions = this.categoryOptions[newIndex].items;
        this.typeSelection = 0;
    }
    componentWillLoad() {
        // If categories have already been provided, set the types options to the
        // items of the first category, as that is selected by default.
        this.typeOptions = this.categoryOptions[0].items;
    }
    componentDidLoad() {
        this.updateCategory(this.categorySelection);
        this.onDetailChanged();
    }
    /**
     * Returns a `Promise` that resolves to a `MapElementDetail` object with
     * values set as those of this `DetailDialogItem`.
     */
    getDetail() {
        return Promise.resolve({
            description: this.description,
            name: this.name,
            code: this.code,
            altText: this.altText,
            id: this.itemId,
            detailTypeId: this.typeOptions[this.typeSelection].id,
        });
    }
    /**
     * Returns a `Promise` that resolves to whether or not this `DetailDialogItem`
     * is to be removed or not.
     */
    toRemove() {
        return Promise.resolve(this.remove);
    }
    hostData() {
        return {
            class: {
                'rl-detail-dialog-item': true,
                'rl-detail-dialog-item--removed': this.remove,
            },
            id: `rl-detail-dialog-item-${this.index}`
        };
    }
    render() {
        // if (this.categorySelection === undefined || this.typeSelection === undefined || this.remove) {
        //   return null;
        // }
        return ([
            h("div", { class: "rl-text-field-container" },
                h("rl-select-menu", { label: "Category", options: this.categoryOptions.map(c => c.name), selectedOption: this.categorySelection, onSelected: evt => {
                        const categories = this.categoryOptions[evt.detail];
                        this.updateCategory(evt.detail);
                        // this.categorySelection = evt.detail;
                        this.typeOptions = [...categories.items];
                        this.typeSelection = 0;
                    } })),
            h("div", { class: "rl-text-field-container" },
                h("rl-select-menu", { label: "Type", options: this.typeOptions.map((i) => i.name), selectedOption: this.typeSelection, onSelected: evt => {
                        this.typeSelection = evt.detail;
                    } })),
            h("div", { class: "rl-text-field-container" },
                h("rl-text-field", { label: "Code", value: this.code, onChangeValue: (e) => this.code = e.detail })),
            h("div", { class: "rl-text-field-container" },
                h("rl-text-field", { label: "Name", value: this.name, onChangeValue: (e) => this.name = e.detail })),
            h("div", { class: "rl-text-field-container" },
                h("rl-text-field", { label: "Description", value: this.description, onChangeValue: (e) => this.description = e.detail })),
            h("div", { class: "rl-text-field-container" },
                h("rl-text-field", { label: "Alt Text", value: this.altText, onChangeValue: (e) => this.altText = e.detail })),
            h("button", { type: "button", class: "mdc-button" },
                h("span", { class: "mdc-button__label", onClick: _ => this.remove = true }, "Remove"))
        ]);
    }
    static get is() { return "rl-detail-dialog-item"; }
    static get originalStyleUrls() { return {
        "$": ["detail-dialog-item.scss"]
    }; }
    static get styleUrls() { return {
        "$": ["detail-dialog-item.css"]
    }; }
    static get properties() { return {
        "detail": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "MapElementDetail",
                "resolved": "MapElementDetail | undefined",
                "references": {
                    "MapElementDetail": {
                        "location": "import",
                        "path": "../../interface"
                    }
                }
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "The `MapElementDetail` that this item is displaying the information of."
            }
        },
        "categoryOptions": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "{ name: string, id: number, items: MapElementDetailType[] }[]",
                "resolved": "{ name: string; id: number; items: MapElementDetailType[]; }[]",
                "references": {
                    "MapElementDetailType": {
                        "location": "import",
                        "path": "../../interface"
                    }
                }
            },
            "required": true,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "An array of all the different categories that can be selected."
            }
        }
    }; }
    static get states() { return {
        "remove": {},
        "description": {},
        "name": {},
        "altText": {},
        "detailTypeId": {},
        "code": {},
        "categorySelection": {},
        "typeOptions": {},
        "typeSelection": {}
    }; }
    static get methods() { return {
        "getDetail": {
            "complexType": {
                "signature": "() => Promise<MapElementDetail>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    },
                    "MapElementDetail": {
                        "location": "import",
                        "path": "../../interface"
                    }
                },
                "return": "Promise<MapElementDetail>"
            },
            "docs": {
                "text": "Returns a `Promise` that resolves to a `MapElementDetail` object with\nvalues set as those of this `DetailDialogItem`.",
                "tags": []
            }
        },
        "toRemove": {
            "complexType": {
                "signature": "() => Promise<boolean>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<boolean>"
            },
            "docs": {
                "text": "Returns a `Promise` that resolves to whether or not this `DetailDialogItem`\nis to be removed or not.",
                "tags": []
            }
        }
    }; }
    static get watchers() { return [{
            "propName": "detail",
            "methodName": "onDetailChanged"
        }, {
            "propName": "categoryOptions",
            "methodName": "onCategoryOptionsChanged"
        }]; }
}
