import { h } from '@stencil/core';
import { MapMarker } from '../../classes/map-marker';
import { MapPolygon } from '../../classes/map-polygon';
import { Coordinate } from '../../utils/coordinate';
import { coordinateFromEvent, encodeCoordinates, } from '../../utils/helpers';
import { DEFAULT_MAX_SCALE, DEFAULT_MIN_SCALE, HYSTERESIS, STATES, computeLimits, getTargetId, parseElements, } from '../../utils/map-base';
export class RLMapEditor {
    constructor() {
        /**
         * The original scale factor used to size the SVG so it fits into the SVG and
         * covers the entire SVG.
         */
        this.initialScale = 1;
        /**
         * The array of MapElements currently being displayed.  Created from the
         * `elements` Prop with additional internal information added.
         */
        this.processedElements = [];
        /**
         * The current state of the Map.
         */
        this.state = STATES.NORMAL;
        /**
         * Flag indicating if the map image has loaded or not.
         */
        this.imageLoaded = false;
        /**
         * The current size of the SVG element used to display the map.
         */
        this.size = { width: 1, height: 1 };
        /**
         * The factor by which the Map contents are changed to fit within the SVG.
         */
        this.svgScale = 1;
        /**
         * The x,y translation applied to move the map contents within the bounds of
         * the SVG.
         */
        this.svgTransform = new Coordinate(0, 0);
        /**
         * An array of the elements that will be displayed on the Map.
         */
        this.elements = {};
        /**
         *  The maximum scale factor.
         */
        this.maxScale = DEFAULT_MAX_SCALE;
        /**
         * The minimum scale factor.
         */
        this.minScale = DEFAULT_MIN_SCALE;
    }
    /**
     * Handle when the list of specified elements changes. The event details
     * contains the `MapElement` that was deleted.
     */
    onElementsChanged() {
        // if (this.elements !== undefined) {
        this.processedElements = parseElements(this.elements, this.svgScale);
        // }
    }
    onMapImageChanged() {
        if (this.mapImage === undefined) {
            return;
        }
        const img = new Image();
        img.src = this.mapImage;
        this.imageLoaded = false;
        img.onload = () => {
            this.imageLoaded = true;
            if (this.imgSize &&
                this.imgSize.width === img.width &&
                this.imgSize.height === img.height) {
                return;
            }
            this.imgSize = { width: img.width, height: img.height };
            this.computeScale();
            this.limits = this.computeLimits();
        };
    }
    componentDidLoad() {
        this.onMapImageChanged();
        this.onElementsChanged();
        this.onResize();
    }
    /**
     * Handles when the user triggers a touch/down event to start interaction with
     * the map.
     * @param e The triggering event.
     */
    onGestureDown(e) {
        this.start = this.last = coordinateFromEvent(e, this.root);
        if (this.state === STATES.NORMAL) {
            this.state = STATES.GESTURE_DOWN;
        }
        if (this.state === STATES.ADD_REGION_INIT) {
            if (this.activeElement && this.activeElement instanceof MapPolygon) {
                this.activeElement.addPoint(this.toSvgSpace(this.start));
                this.root.forceUpdate();
            }
            this.state = STATES.ADD_REGION_FIRST;
        }
        else if (this.state !== STATES.ADD_POINT) {
            // Get the ID of the event target, if it is the correct type.
            const id = getTargetId(e.target);
            if (id !== undefined) {
                this.targetElement = this.processedElements.find(i => i.id === id);
            }
            if (e.target && e.target instanceof SVGCircleElement) {
                if (e.target.classList.contains('rl-svg__control') ||
                    e.target.classList.contains('rl-svg__midpoint')) {
                    this.targetControl = Number(e.target.dataset.index);
                }
            }
        }
    }
    /**
     * Handles when the user triggers a movement event.
     * @param e The triggering event.
     */
    onGestureMove(e) {
        if (this.state === STATES.NORMAL || this.state === STATES.ADD_REGION_INIT) {
            return;
        }
        // Convert the coordinate to SVG coordinate space.
        const point = coordinateFromEvent(e, this.root);
        const delta = Coordinate.difference(point, this.last);
        let dist = 0;
        switch (this.state) {
            case STATES.GESTURE_DOWN:
                if (this.start && (this.targetControl !== undefined || this.activeElement && this.targetElement === this.activeElement)) {
                    dist = Coordinate.squareDistance(this.start, point);
                    if (dist > HYSTERESIS) {
                        this.state = STATES.DRAGGING;
                    }
                    break;
                }
            case STATES.DRAGGING:
                if (this.targetControl !== undefined && this.activeElement !== undefined && this.activeElement instanceof MapPolygon) {
                    if (this.targetControl % 2 === 1) {
                        // If the target control is a midpoint, Add it to the array of
                        // controls.
                        const i = Math.floor(this.targetControl / 2) + 1;
                        this.activeElement.addPoint(Coordinate.difference(this.toSvgSpace(point), this.toSvgSpace(this.svgTransform)), i);
                        // Update the targetControl to be the index of the new control.
                        this.targetControl = i * 2;
                    }
                    else {
                        this.activeElement.movePoint(this.toSvgScale(delta), this.targetControl / 2);
                        this.root.forceUpdate();
                    }
                }
                else if (this.targetElement && this.activeElement &&
                    this.activeElement.id === this.targetElement.id) {
                    this.targetElement.move(this.toSvgScale(delta));
                    this.root.forceUpdate();
                }
                else {
                    // Move the canvas otherwise.
                    if (this.limits !== undefined) {
                        this.state = STATES.DRAGGING;
                        this.svgTransform = Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                    }
                }
                break;
            case STATES.ADD_REGION_FIRST:
                if (this.start) {
                    dist = Coordinate.squareDistance(this.start, point);
                    if (this.activeElement && this.activeElement instanceof MapPolygon && dist > HYSTERESIS) {
                        this.state = STATES.ADD_REGION;
                        this.activeElement.addPoint(this.toSvgSpace(point));
                        this.root.forceUpdate();
                    }
                }
                break;
            case STATES.ADD_REGION:
                if (this.activeElement && this.activeElement instanceof MapPolygon) {
                    const idx = this.activeElement.points.length - 1;
                    this.activeElement.movePoint(this.toSvgScale(delta), idx);
                    this.root.forceUpdate();
                }
                break;
            default:
        }
        this.last = point;
    }
    /**
     * Handles when the user ends their interaction with the Map.
     * @param e The triggering event.
     */
    onGestureUp(e) {
        // Convert the coordinate to SVG coordinate space.
        const point = coordinateFromEvent(e, this.root);
        switch (this.state) {
            case STATES.GESTURE_DOWN:
                if (this.targetElement) {
                    if (this.targetElement !== this.activeElement) {
                        this._setActiveElement(this.targetElement);
                    }
                }
                else {
                    this._clearActiveElement();
                    this.elementDeselected.emit(this.activeElement);
                }
                this.state = STATES.NORMAL;
                break;
            case STATES.DRAGGING:
                const delta = Coordinate.difference(point, this.last);
                if (this.activeElement && this.activeElement instanceof MapPolygon && this.targetControl !== undefined) {
                    this.activeElement.movePoint(this.toSvgScale(delta), this.targetControl / 2);
                    this.elementUpdated.emit(this.mapElementFromParsedElement(this.activeElement));
                }
                else if (this.targetElement && this.activeElement &&
                    this.activeElement.id === this.targetElement.id) {
                    // Move the targeted element, if there is one.
                    this.activeElement.move(this.toSvgScale(delta));
                    this.elementUpdated.emit(this.mapElementFromParsedElement(this.activeElement));
                }
                else {
                    // Move the canvas otherwise.
                    if (this.limits) {
                        this.svgTransform = Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                    }
                }
                this.state = STATES.NORMAL;
                break;
            case STATES.ADD_REGION_FIRST:
                if (this.activeElement && this.activeElement instanceof MapPolygon) {
                    this.activeElement.addPoint(this.toSvgSpace(point));
                    this.state = STATES.ADD_REGION;
                }
                break;
            case STATES.ADD_REGION:
                if (this.activeElement && this.activeElement instanceof MapPolygon) {
                    if (this.targetControl === 0) {
                        // Remove the last point, it was used as a placeholder for drawing
                        // a line to the cursor.
                        this.activeElement.removePoint();
                        this.state = STATES.NORMAL;
                        this.root.forceUpdate();
                        this.elementCreated.emit(this.mapElementFromParsedElement(this.activeElement));
                    }
                    else {
                        // Add a new point to the active controls array.
                        this.activeElement.addPoint(this.toSvgSpace(point));
                    }
                }
                break;
            case STATES.ADD_POINT:
                if (this.activeElement && this.activeElement instanceof MapMarker && this.root) {
                    this.activeElement.position = this.toSvgSpace(point);
                    this.state = STATES.NORMAL;
                    this.root.forceUpdate();
                    this.elementCreated.emit(this.mapElementFromParsedElement(this.activeElement));
                }
            default:
        }
        this.targetControl = undefined;
        this.targetElement = undefined;
    }
    /**
     * Handles when the user leaves the bounds of the Map entirely.
     * @param e The triggering event.
     */
    onMouseLeave(e) {
        if (this.state === STATES.DRAGGING ||
            this.state === STATES.GESTURE_DOWN) {
            this.state = STATES.NORMAL;
            const point = coordinateFromEvent(e, this.root);
            if (this.state === STATES.DRAGGING && this.activeElement) {
                const delta = Coordinate.difference(point, this.last);
                if (this.activeElement && this.activeElement instanceof MapPolygon && this.targetControl !== undefined) {
                    this.activeElement.movePoint(this.toSvgScale(delta), this.targetControl);
                    this.elementUpdated.emit(this.mapElementFromParsedElement(this.activeElement));
                }
                else if (this.targetElement && this.activeElement &&
                    this.activeElement.id === this.targetElement.id) {
                    // Move the targeted element, if there is one.
                    this.activeElement.move(this.toSvgScale(delta));
                    this.elementUpdated.emit(this.mapElementFromParsedElement(this.activeElement));
                }
                else {
                    // Move the canvas otherwise.
                    if (this.limits !== undefined) {
                        this.svgTransform = Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                    }
                }
                this.state = STATES.NORMAL;
            }
        }
    }
    onDoubleClick(e) {
        if (this.state === STATES.NORMAL) {
            const id = getTargetId(e.target);
            if (id !== undefined) {
                this.elementDoubleClicked.emit(this.elements[id]);
            }
        }
    }
    /**
     * Handles when the window is resized.
     */
    onResize() {
        clearTimeout(this.resizeDebounce);
        this.resizeDebounce = setTimeout(_ => {
            // Use a 150ms delay to debounce multiple resize calls and effectively
            // perform the resize action only when the user is 'done' resizing.
            this.computeSvgSize();
            this.computeScale();
            this.limits = this.computeLimits();
            this.svgTransform = this.svgTransform.clone().limit(this.limits);
        }, 150);
    }
    /**
     * Handles when the user performs a mouse wheen event.  The Map will be zoomed
     * in or out depending on the direction of the wheel.
     *
     * @param e The triggering event.
     */
    onWheel(e) {
        const oldPos = this.toSvgSpace(new Coordinate(e.clientX, e.clientY));
        if (e.deltaY < 0) {
            if (this.svgScale * 1.05263157895 < this.initialScale * this.maxScale) {
                this.svgScale *= 1.05263157895;
            }
        }
        else {
            if (this.svgScale * 0.95 > this.initialScale * this.minScale) {
                this.svgScale *= 0.95;
            }
        }
        this.limits = this.computeLimits();
        this.processedElements.forEach(el => el.scale = this.svgScale);
        const newPos = this.toSvgSpace(new Coordinate(e.clientX, e.clientY));
        const delta = Coordinate.difference(newPos, oldPos).scale(this.svgScale);
        this.svgTransform.translate(delta).limit(this.limits).round();
    }
    /**
     * Handle when the user pressses the Enter key.
     * @param e The triggering event
     */
    onEnter(e) {
        if (e.key === 'enter' && e.target && e.target instanceof SVGElement &&
            (e.target.classList.contains('rl-svg__polygon') ||
                e.target.classList.contains('rl-svg__marker'))) {
            const id = Number(e.target.id);
            const el = this.processedElements.find(i => i.id === id);
            this._setActiveElement(el);
        }
    }
    /**
     * Add a new `MapRegion` to the map. Calling this method starts the process.
     * The user must then click numerous times on the map to add points.  Only
     * when clicking again on the original point is the region added.
     */
    async addRegion() {
        this._clearActiveElement();
        this.state = STATES.ADD_REGION_INIT;
        this.activeElement = new MapPolygon();
        this.activeElement.open = true;
        this.activeElement.scale = this.svgScale;
        this.processedElements = [...this.processedElements, this.activeElement];
    }
    /**
     * Add a new `MapPoint` element to the map.  Calling this method starts the
     * process.  The user must then click somewhere on the map to add the point.
     */
    async addPoint() {
        this._clearActiveElement();
        this.state = STATES.ADD_POINT;
        this.activeElement = new MapMarker();
        this.processedElements = [...this.processedElements, this.activeElement];
    }
    /**
     * Cancels the current action and returns the map to its default state, ready
     * for futher action.
     */
    async cancelAction() {
        this._clearActiveElement();
        this.state = STATES.NORMAL;
    }
    /**
     * Removes the currently active element.  If no element is selected when
     * this method is called, it has no effect.
     */
    async deleteRegion() {
        if (this.state === STATES.NORMAL && this.activeElement !== undefined) {
            this.elementDeleted.emit(this.elements[this.activeElement.id]);
            this._clearActiveElement();
        }
    }
    /**
     * Sets the element with the specified ID to active.
     *
     * @param id The ID of the element to set as active.
     */
    async setActiveElement(id) {
        if (this.processedElements.length > 0) {
            this._setActiveElement(this.processedElements.find(i => i.id === id));
        }
    }
    /**
     * Generates a `MapElement` object based on the values in the corresponding
     * `ParsedElement` object.
     *
     * @param el The ParsedElement to convert back to a MapElement
     */
    mapElementFromParsedElement(el) {
        const originalEl = this.elements[el.id] || {
            altText: '',
            description: '',
            details: undefined,
            floorId: 0,
            iconPath: [],
            enabled: true,
        };
        return {
            altText: originalEl.altText,
            description: originalEl.description,
            details: originalEl.details,
            enabled: originalEl.enabled,
            floorId: originalEl.floorId,
            id: el.id,
            icon: originalEl.icon,
            symbol: originalEl.symbol,
            name: el.name,
            points: el instanceof MapPolygon ? encodeCoordinates(el.points) : encodeCoordinates(el.position),
        };
    }
    /**
     * Clears the currently active element.
     */
    _clearActiveElement() {
        if (this.activeElement) {
            this.activeElement.active = false;
        }
        this.activeElement = undefined;
    }
    /**
     * Updates the limits used to restrict the how far the SVG transform can be
     * moved keeping the map from being panning beyond the edge of the image.
     */
    computeLimits() {
        const { imgSize, size } = this;
        if (imgSize !== undefined) {
            return computeLimits(imgSize, size, this.svgScale);
        }
        else {
            return new DOMRect(0, 0, 1, 1);
        }
    }
    /**
     * Computes the scale factor needed to resize the full map image to be as
     * large as possible, and cropping it vertically/horizontally so no empty
     * space exists if the container has different aspect from the image. This is
     * like CSS `background-size: cover` or SVG `aspectRatio xMidYMid slice`.
     */
    computeScale() {
        const { imgSize, size } = this;
        if (imgSize === undefined) {
            this.svgScale = this.initialScale = 1;
        }
        else {
            this.svgScale = this.initialScale =
                Math.max(size.width / imgSize.width, size.height / imgSize.height);
        }
        this.processedElements.forEach(el => el.scale = this.svgScale);
    }
    computeSvgSize() {
        const svg = this.root.querySelector('svg');
        if (svg) {
            const rect = svg.getBoundingClientRect();
            this.size = { width: rect.width, height: rect.height };
        }
    }
    /**
     * Sets the currently active element of the Map.
     *
     * @param el The Element that will be set as the active element.
     */
    _setActiveElement(el) {
        if (!el) {
            this._clearActiveElement();
            this.elementDeselected.emit();
            return;
        }
        this._clearActiveElement();
        this.activeElement = el;
        this.activeElement.active = true;
        // if (this.elements !== undefined) {
        this.elementSelected.emit(this.elements[this.activeElement.id]);
        // }
    }
    /**
     * Scales a given `Coordinate` by the inverse of the current `svgScale`.
     * Returns a new `Coordinate` with its values scaled and rounded.
     * @param c A `Coordinate` scale.
     */
    toSvgScale(c) {
        return c.clone().scale(1 / this.svgScale).round();
    }
    /**
     * Converts a given `Coordinate` object in screen coordinate space to SVG
     * coordinate space.
     * @param c A `Coordinate` to convert to SVG coordinate space.
     */
    toSvgSpace(c) {
        return Coordinate.difference(this.toSvgScale(c), this.toSvgScale(this.svgTransform));
    }
    hostData() {
        return {
            class: {
                'rl-map-editor': true,
            },
        };
    }
    render() {
        const m = [
            this.svgScale, 0, 0, this.svgScale,
            this.svgTransform.x,
            this.svgTransform.y,
        ];
        const matrix = 'matrix(' + m.join(',') + ')';
        const elements = this.processedElements.map(el => el.render());
        const controls = this.activeElement !== undefined && this.activeElement instanceof MapPolygon && this.activeElement.renderControls();
        return (h("svg", { class: "rl-svg" },
            h("g", { transform: matrix },
                h("image", { xlinkHref: this.mapImage !== undefined ? this.mapImage : undefined }),
                controls ? [elements, controls] : elements)));
    }
    static get is() { return "rl-map-editor"; }
    static get originalStyleUrls() { return {
        "$": ["map-editor.scss"]
    }; }
    static get styleUrls() { return {
        "$": ["map-editor.css"]
    }; }
    static get properties() { return {
        "elements": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "MapElementDataMap",
                "resolved": "MapElementDataMap",
                "references": {
                    "MapElementDataMap": {
                        "location": "import",
                        "path": "../../interface"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "An array of the elements that will be displayed on the Map."
            },
            "defaultValue": "{}"
        },
        "mapImage": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string | undefined",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "The image displayed on the Map."
            },
            "attribute": "map-image",
            "reflect": false
        },
        "maxScale": {
            "type": "number",
            "mutable": false,
            "complexType": {
                "original": "number",
                "resolved": "number",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The maximum scale factor."
            },
            "attribute": "max-scale",
            "reflect": false,
            "defaultValue": "DEFAULT_MAX_SCALE"
        },
        "minScale": {
            "type": "number",
            "mutable": false,
            "complexType": {
                "original": "number",
                "resolved": "number",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The minimum scale factor."
            },
            "attribute": "min-scale",
            "reflect": false,
            "defaultValue": "DEFAULT_MIN_SCALE"
        }
    }; }
    static get states() { return {
        "activeElement": {},
        "imageLoaded": {},
        "size": {},
        "svgScale": {},
        "svgTransform": {}
    }; }
    static get events() { return [{
            "method": "elementCreated",
            "name": "elementCreated",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "An event fired when a new `MapElement` is created. The event details\ncontains the `MapElement` that was created."
            },
            "complexType": {
                "original": "MapElementData",
                "resolved": "MapElementData",
                "references": {
                    "MapElementData": {
                        "location": "import",
                        "path": "../../interface"
                    }
                }
            }
        }, {
            "method": "elementDeselected",
            "name": "elementDeselected",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "An event fired when the user deselects a `MapElement`."
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }, {
            "method": "elementSelected",
            "name": "elementSelected",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "An event fired when the user selects a MapElement. The clicked element\nwill be passed as the event parameter."
            },
            "complexType": {
                "original": "MapElementData",
                "resolved": "MapElementData",
                "references": {
                    "MapElementData": {
                        "location": "import",
                        "path": "../../interface"
                    }
                }
            }
        }, {
            "method": "elementUpdated",
            "name": "elementUpdated",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "An event fired when a `MapElement` is updated (moved or changes shape).\nThe event details contains the `MapElement` that was updated."
            },
            "complexType": {
                "original": "MapElementData",
                "resolved": "MapElementData",
                "references": {
                    "MapElementData": {
                        "location": "import",
                        "path": "../../interface"
                    }
                }
            }
        }, {
            "method": "elementDeleted",
            "name": "elementDeleted",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "An event fired when one of the `MapElements` on this map is deleted."
            },
            "complexType": {
                "original": "MapElementData",
                "resolved": "MapElementData",
                "references": {
                    "MapElementData": {
                        "location": "import",
                        "path": "../../interface"
                    }
                }
            }
        }, {
            "method": "elementDoubleClicked",
            "name": "elementDoubleClicked",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "An event fired when one of the `MapElement`s on the map is double clicked."
            },
            "complexType": {
                "original": "MapElementData",
                "resolved": "MapElementData",
                "references": {
                    "MapElementData": {
                        "location": "import",
                        "path": "../../interface"
                    }
                }
            }
        }]; }
    static get methods() { return {
        "addRegion": {
            "complexType": {
                "signature": "() => Promise<void>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<void>"
            },
            "docs": {
                "text": "Add a new `MapRegion` to the map. Calling this method starts the process.\nThe user must then click numerous times on the map to add points.  Only\nwhen clicking again on the original point is the region added.",
                "tags": []
            }
        },
        "addPoint": {
            "complexType": {
                "signature": "() => Promise<void>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<void>"
            },
            "docs": {
                "text": "Add a new `MapPoint` element to the map.  Calling this method starts the\nprocess.  The user must then click somewhere on the map to add the point.",
                "tags": []
            }
        },
        "cancelAction": {
            "complexType": {
                "signature": "() => Promise<void>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<void>"
            },
            "docs": {
                "text": "Cancels the current action and returns the map to its default state, ready\nfor futher action.",
                "tags": []
            }
        },
        "deleteRegion": {
            "complexType": {
                "signature": "() => Promise<void>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<void>"
            },
            "docs": {
                "text": "Removes the currently active element.  If no element is selected when\nthis method is called, it has no effect.",
                "tags": []
            }
        },
        "setActiveElement": {
            "complexType": {
                "signature": "(id: number) => Promise<void>",
                "parameters": [{
                        "tags": [{
                                "text": "id The ID of the element to set as active.",
                                "name": "param"
                            }],
                        "text": "The ID of the element to set as active."
                    }],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<void>"
            },
            "docs": {
                "text": "Sets the element with the specified ID to active.",
                "tags": [{
                        "name": "param",
                        "text": "id The ID of the element to set as active."
                    }]
            }
        }
    }; }
    static get elementRef() { return "root"; }
    static get watchers() { return [{
            "propName": "elements",
            "methodName": "onElementsChanged"
        }, {
            "propName": "mapImage",
            "methodName": "onMapImageChanged"
        }]; }
    static get listeners() { return [{
            "name": "mousedown",
            "method": "onGestureDown",
            "target": undefined,
            "capture": false,
            "passive": true
        }, {
            "name": "touchstart",
            "method": "onGestureDown",
            "target": undefined,
            "capture": false,
            "passive": true
        }, {
            "name": "mousemove",
            "method": "onGestureMove",
            "target": undefined,
            "capture": false,
            "passive": true
        }, {
            "name": "touchmove",
            "method": "onGestureMove",
            "target": undefined,
            "capture": false,
            "passive": true
        }, {
            "name": "mouseup",
            "method": "onGestureUp",
            "target": undefined,
            "capture": false,
            "passive": true
        }, {
            "name": "touchend",
            "method": "onGestureUp",
            "target": undefined,
            "capture": false,
            "passive": true
        }, {
            "name": "mouseleave",
            "method": "onMouseLeave",
            "target": undefined,
            "capture": false,
            "passive": true
        }, {
            "name": "dblclick",
            "method": "onDoubleClick",
            "target": undefined,
            "capture": false,
            "passive": false
        }, {
            "name": "resize",
            "method": "onResize",
            "target": "window",
            "capture": false,
            "passive": true
        }, {
            "name": "wheel",
            "method": "onWheel",
            "target": undefined,
            "capture": false,
            "passive": true
        }, {
            "name": "keydown",
            "method": "onEnter",
            "target": undefined,
            "capture": false,
            "passive": false
        }]; }
}
