import { MDCTextField } from '@material/textfield/index';
import { h } from '@stencil/core';
let nextId = 0;
/**
 * A component that wraps a `Material Text Field` component.
 */
export class TextField {
    constructor() {
        /**
         * Internal ID used to differentiate multiple text fields.
         */
        this.id = nextId++;
        /**
         * The supplemental label for the text field.
         */
        this.label = '';
        /**
         * A flag indicating if the text field has an outlined style.
         */
        this.outlined = false;
        /**
         * A flag indicating if the text field is disabled and does not allow user
         * input.
         */
        this.disabled = false;
        /**
         * A flag indicating if the text field uses the full-width style.
         */
        this.fullwidth = false;
        /**
         * A flag indicating if the text field uses a textarea instead of an input.
         */
        this.textarea = false;
        /**
         * Additional text displayed below the main text field.
         */
        this.helperText = '';
        /**
         * The current value of the text field input or text area.
         */
        this.value = '';
        /**
         * An icon displayed within the text field.
         */
        this.icon = '';
        /**
         * The location of the icon displayed within the text field.
         */
        this.iconLocation = 'leading';
    }
    componentDidLoad() {
        new MDCTextField(this.root);
    }
    hostData() {
        return {
            class: {
                'rl-text-field': true,
                'mdc-text-field': true,
                'mdc-text-field--outlined': this.outlined,
                'mdc-text-field--disabled': this.disabled,
                'mdc-text-field--fullwidth': this.fullwidth,
                'mdc-text-field--textarea': this.textarea,
                'mdc-text-field--with-leading-icon': this.icon !== '' && this.iconLocation === 'leading',
                'mdc-text-field--with-trailing-icon': this.icon !== '' && this.iconLocation === 'trailing',
            }
        };
    }
    render() {
        const fieldId = `rl-text-field-${this.id}`;
        const label = (h("label", { htmlFor: fieldId, class: `mdc-floating-label ${this.value !== '' ? 'mdc-floating-label--float-above' : ''}` }, this.label));
        const icon = this.icon !== '' ?
            (h("i", { class: "material-icons mdc-text-field__icon" }, this.icon)) :
            undefined;
        const field = this.textarea ?
            (h("textarea", { id: "textarea", class: "mdc-text-field__input", rows: 8, cols: 40, onChange: e => {
                    if (e.target !== null &&
                        (e.target instanceof HTMLInputElement ||
                            e.target instanceof HTMLTextAreaElement)) {
                        this.changeValue.emit(e.target.value);
                    }
                } }, this.value)) :
            (h("input", { type: "text", id: fieldId, class: "mdc-text-field__input", value: this.value, onChange: e => {
                    if (e.target !== null &&
                        (e.target instanceof HTMLInputElement ||
                            e.target instanceof HTMLTextAreaElement)) {
                        this.changeValue.emit(e.target.value);
                    }
                } }));
        const content = this.textarea ?
            (h("div", { class: `mdc-notched-outline ${this.value !== '' ? 'mdc-notched-outline--notched' : ''}` },
                h("div", { class: "mdc-notched-outline__leading" }),
                h("div", { class: "mdc-notched-outline__notch" }, label),
                h("div", { class: "mdc-notched-outline__trailing" }))) : this.outlined ?
            (h("div", { class: `mdc-notched-outline ${this.value !== '' ? 'mdc-notched-outline--notched' : ''}` },
                h("div", { class: "mdc-notched-outline__leading" }),
                h("div", { class: "mdc-notched-outline__notch" }, label),
                h("div", { class: "mdc-notched-outline__trailing" }))) :
            ([label, h("div", { class: "mdc-line-ripple" })]);
        return ([
            icon,
            field,
            content
        ]);
    }
    static get is() { return "rl-text-field"; }
    static get originalStyleUrls() { return {
        "$": ["text-field.scss"]
    }; }
    static get styleUrls() { return {
        "$": ["text-field.css"]
    }; }
    static get properties() { return {
        "label": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The supplemental label for the text field."
            },
            "attribute": "label",
            "reflect": false,
            "defaultValue": "''"
        },
        "outlined": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "A flag indicating if the text field has an outlined style."
            },
            "attribute": "outlined",
            "reflect": false,
            "defaultValue": "false"
        },
        "disabled": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "A flag indicating if the text field is disabled and does not allow user\ninput."
            },
            "attribute": "disabled",
            "reflect": false,
            "defaultValue": "false"
        },
        "fullwidth": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "A flag indicating if the text field uses the full-width style."
            },
            "attribute": "fullwidth",
            "reflect": false,
            "defaultValue": "false"
        },
        "textarea": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "A flag indicating if the text field uses a textarea instead of an input."
            },
            "attribute": "textarea",
            "reflect": false,
            "defaultValue": "false"
        },
        "helperText": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Additional text displayed below the main text field."
            },
            "attribute": "helper-text",
            "reflect": false,
            "defaultValue": "''"
        },
        "value": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The current value of the text field input or text area."
            },
            "attribute": "value",
            "reflect": false,
            "defaultValue": "''"
        },
        "icon": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "An icon displayed within the text field."
            },
            "attribute": "icon",
            "reflect": false,
            "defaultValue": "''"
        },
        "iconLocation": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "'trailing' | 'leading'",
                "resolved": "\"leading\" | \"trailing\"",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The location of the icon displayed within the text field."
            },
            "attribute": "icon-location",
            "reflect": false,
            "defaultValue": "'leading'"
        }
    }; }
    static get events() { return [{
            "method": "changeValue",
            "name": "changeValue",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "An event emitted when the value of the input or textarea changes."
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }]; }
    static get elementRef() { return "root"; }
}
