import { f as h } from './rlwc-ca835c31.js';

/**
 * Class for representing coordinates and positions.
 * Adapted from Google Closure Library.
 */
class Coordinate {
    /**
     * Create a new Coordinate.
     */
    constructor(x, y) {
        this.x = x;
        this.y = y;
    }
    /**
     * Returns a new copy of the coordinate.
     */
    clone() {
        return new Coordinate(this.x, this.y);
    }
    /**
     * Returns a nice string representation the coordinate.
     */
    toString() {
        return '(' + this.x + ', ' + this.y + ')';
    }
    /**
     * Returns a basic string representation of the Coordinate used to easily
     * create SVG path strings.
     */
    toPathString() {
        return this.x + ' ' + this.y;
    }
    /**
     * Returns whether the specified value is equal to this coordinate.
     * @param other Some other value.
     */
    equals(other) {
        return other instanceof Coordinate && Coordinate.equals(this, other);
    }
    /**
     * Rounds the x and y fields to the next larger integer values.
     */
    ceil() {
        this.x = Math.ceil(this.x);
        this.y = Math.ceil(this.y);
        return this;
    }
    /**
     * Rounds the x and y fields to the next smaller integer values.
     */
    floor() {
        this.x = Math.floor(this.x);
        this.y = Math.floor(this.y);
        return this;
    }
    /**
     * Translates this coordinate by the given offsets. If a `Coordinate`
     * is given, then the x and y values are translated by the coordinate's x
     * and y. Otherwise, x and y are translated by `tx` and `ty` respectively.
     *
     * @param tx The value to translate x by or the
     *     the coordinate to translate this coordinate by.
     * @param ty The value to translate y by.
     */
    translate(tx, ty) {
        if (tx instanceof Coordinate) {
            this.x += tx.x;
            this.y += tx.y;
        }
        else {
            this.x += Number(tx);
            if (typeof ty === 'number') {
                this.y += ty;
            }
        }
        return this;
    }
    /**
     * Scales this coordinate by the given scale factors. The x and y values are
     * scaled by `sx` and `sy` respectively.  If `sy` is not given, then `sx` is
     * used for both x and y.
     *
     * @param sx The scale factor to use for the x dimension.
     * @param opt_sy The scale factor to use for the y dimension.
     */
    scale(sx, sy) {
        sy = typeof sy === 'number' ? sy : sx;
        this.x *= sx;
        this.y *= sy;
        return this;
    }
    /**
     * Rotates this coordinate clockwise about the origin (or, optionally, the
     * given center) by the given angle, in radians.
     *
     * @param radians The angle by which to rotate this coordinate
     *     clockwise about the given center, in radians.
     * @param center The center of rotation. Defaults
     *     to (0, 0) if not given.
     */
    rotateRadians(radians, center) {
        center = center || new Coordinate(0, 0);
        const x = this.x;
        const y = this.y;
        const cos = Math.cos(radians);
        const sin = Math.sin(radians);
        this.x = (x - center.x) * cos - (y - center.y) * sin + center.x;
        this.y = (x - center.x) * sin + (y - center.y) * cos + center.y;
        return this;
    }
    /**
     * Rotates this coordinate clockwise about the origin (or, optionally, the
     * given center) by the given angle, in degrees.
     *
     * @param degrees The angle by which to rotate this coordinate
     *     clockwise about the given center, in degrees.
     * @param opt_center The center of rotation. Defaults
     *     to (0, 0) if not given.
     */
    rotateDegrees(degrees, center) {
        return this.rotateRadians(degrees * Math.PI / 180, center);
    }
    /**
     * Restricts each parameter of the Coordinate such that
     * `left <= x <= left + width` and `top <= y <= top + height`.
     *
     * @param limits A rectangle to restrict the Coordinate to.
     */
    limit(limits) {
        // Limit in the x direction.
        const left = !isNaN(limits.left) ? limits.left : undefined;
        const width = !isNaN(limits.width) ? limits.width : 0;
        const maxX = left !== undefined ? left + width : Infinity;
        const minX = left !== undefined ? left : -Infinity;
        this.x = Math.min(maxX, Math.max(minX, this.x));
        // Limit in the y direction.
        const top = !isNaN(limits.top) ? limits.top : undefined;
        const height = !isNaN(limits.height) ? limits.height : 0;
        const maxY = top !== undefined ? top + height : Infinity;
        const minY = top !== undefined ? top : -Infinity;
        this.y = Math.min(maxY, Math.max(minY, this.y));
        return this;
    }
    /**
     * Round the values of the `Coordinate` to a given precision (number of
     * decimals).  Defaults to a precision of 0, or rounding to the nearest
     * integer.
     *
     * @param precision The number of decimals to round to
     */
    round(precision = 0) {
        const factor = Math.pow(10, precision);
        this.x = Math.round(this.x * factor) / factor;
        this.y = Math.round(this.y * factor) / factor;
        return this;
    }
    /**
     * Compares two `Coordinate`s for equlaity.
     * @param a The first `Coordinate`.
     * @param b The second `Coordinate`.
     */
    static equals(a, b) {
        if (a === b) {
            return true;
        }
        return a.x === b.x && a.y === b.y;
    }
    /**
     * Returns the distance between two `Coordinate`s.
     * @param a The first `Coordinate`.
     * @param b The second `Coordinate`.
     */
    static distance(a, b) {
        const dx = a.x - b.x;
        const dy = a.y - b.y;
        return Math.sqrt(dx * dx + dy * dy);
    }
    /**
     * Returns the magnitude (distance from origin) of a `Coordinate`.
     * @param a The first `Coordinate`.
     */
    static magnitude(a) {
        return Math.sqrt(a.x * a.x + a.y * a.y);
    }
    /**
     * Returns the squared distance between two `Coordinate`s. Squared distances
     * can be used for comparisons when the actual value is not required.
     *
     * Performance note: eliminating the square root is an optimization often used
     * in lower-level languages, but the speed difference is not nearly as
     * pronounced in JavaScript (only a few percent.)
     *
     * @param a The first `Coordinate`.
     * @param b The second `Coordinate`.
     */
    static squareDistance(a, b) {
        const dx = a.x - b.x;
        const dy = a.y - b.y;
        return dx * dx + dy * dy;
    }
    /**
     * Returns the difference between two `Coordinate`s.
     * @param a The first `Coordinate`.
     * @param b The second `Coordinate`.
     */
    static difference(a, b) {
        if (!b) {
            return new Coordinate(0, 0);
        }
        return new Coordinate(a.x - b.x, a.y - b.y);
    }
    /**
     * Returns the sum of two coordinates as a new `Coordinate`.
     * @param a The first `Coordinate`.
     * @param b The second `Coordinate`.
     */
    static sum(a, b) {
        return new Coordinate(a.x + b.x, a.y + b.y);
    }
    /**
     * Returns the midpoint of two coordinates as a new `Coordinate`.
     * @param a The first `Coordinate`.
     * @param b The second `Coordinate`.
     */
    static midpoint(a, b) {
        return new Coordinate((a.x + b.x) / 2, (a.y + b.y) / 2);
    }
}

/**
 * Returns the position of the event or the element's border box relative to
 * the client viewport. If an event is passed, and if this event is a "touch"
 * event, then the position of the first changedTouches will be returned.
 *
 * @param el Element or a mouse / touch event.
 * @return The position.
 */
function getClientPosition(el) {
    if (el.nodeType === 1 /* NodeType ELEMENT */) {
        return getClientPositionForElement_(el);
    }
    else {
        const targetEvent = el.changedTouches ? el.changedTouches[0] : el;
        return new Coordinate(targetEvent.clientX, targetEvent.clientY);
    }
}
/**
 * Returns the position of the event or the element's border box relative to
 * the client viewport.
 *
 * @param el Element whose position to get.
 * @return The position.
 */
function getClientPositionForElement_(el) {
    const box = getBoundingClientRect_(el);
    return new Coordinate(box.left, box.top);
}
/**
 * Gets the client rectangle of the DOM element.
 *
 * getBoundingClientRect is part of a new CSS object model draft (with a
 * long-time presence in IE), replacing the error-prone parent offset
 * computation and the now-deprecated Gecko getBoxObjectFor.
 *
 * This utility patches common browser bugs in getBoundingClientRect. It
 * will fail if getBoundingClientRect is unsupported.
 *
 * If the element is not in the DOM, the result is undefined, and an error may
 * be thrown depending on user agent.
 *
 * @param el The element whose bounding rectangle is being queried.
 * @return A native bounding rectangle with numerical left, top,
 *     right, and bottom.  Reported by Firefox to be of object type ClientRect.
 */
function getBoundingClientRect_(el) {
    let rect;
    try {
        rect = el.getBoundingClientRect();
    }
    catch (e) {
        // In IE < 9, calling getBoundingClientRect on an orphan element raises an
        // "Unspecified Error". All other browsers return zeros.
        return { 'left': 0, 'top': 0, 'right': 0, 'bottom': 0 };
    }
    return rect;
}
/**
 * Creates a new Coordinate based on the clientX/Y values of a given event or the
 * top-left corner of an element.  Optionally gets the position relative to a
 * second element.
 *
 * @param e The event to retrieve the client coordinates from.
 */
function coordinateFromEvent(a, b) {
    const ap = getClientPosition(a);
    const bp = getClientPosition(b);
    return new Coordinate(ap.x - bp.x, ap.y - bp.y);
}
/**
 * Parses a string of x/y coordinate pairs separated by spaces.  Returns an
 * object with two properties.  The `points` property is an array of Coordinate
 * objects representing each x/y pair and the `path` property is an SVG encoded
 * path string in the form `M x1 y1 L x2 y2 ... L xN yN Z`.
 *
 * @param data A string of space-separated coordinates.
 */
function decodeCoordinates(data, closed) {
    const pointData = data.split(' ');
    let points = [];
    let i = pointData.length - 2;
    let x;
    let y;
    // Closed paths end with 'Z'. Start with that and work backwards unless open.
    let path = closed ? 'Z' : '';
    while (i >= 0) {
        x = Math.round(Number(pointData[i]));
        y = Math.round(Number(pointData[i + 1]));
        points.push(new Coordinate(x, y));
        path = (i === 0 ? ['M', x, y, path].join(' ') : ['L', x, y, path].join(' '));
        i -= 2;
    }
    if (points.length === 1) {
        points = points[0];
    }
    return { path, points };
}
/**
 * Turns an array of Coordinate objects into a single space-separated string.
 * @param points An array of Coordinates.
 */
function encodeCoordinates(points) {
    if (points instanceof Coordinate) {
        return points.toPathString();
    }
    else {
        return points.map(i => {
            return i.toPathString();
        }).join(' ');
    }
}
/**
 * Generates an SVG-style path string from an array of `Coordinates`.
 *
 * @param points An array of `Coordinates`.
 * @param open If the path represented by the array of points is open or closed.
 */
function pathFromCoordinateArray(points, open = false) {
    return points.map((i, idx) => {
        if (idx === 0) {
            return 'M ' + i.toPathString();
        }
        else if (idx === points.length - 1 && !open) {
            return ' L ' + i.toPathString() + ' Z';
        }
        else {
            return ' L ' + i.toPathString();
        }
    }).join('');
}

/**
 * The base class for the different kinds of elements that can be rendered on a
 * Map.
 */
class MapElement {
    /**
     * Create a new `MapElement`.
     *
     * @param id A value to set as the ID of the `MapElement`.
     */
    constructor(opts) {
        this._active = false;
        this._clickable = true;
        this._id = -1;
        this._name = '';
        this._scale = 1;
        this._visible = true;
        this._zIndex = 1;
        if (opts) {
            this.name = (opts.name !== undefined) ? opts.name : '';
            this.zIndex = (opts.zIndex !== undefined) ? opts.zIndex : 1;
        }
    }
    /**
     * Indicates whether this `MapElement` is currently selected by the user. When
     * active, the `MapElement` is styled differently.  Defaults to `false`.
     */
    set active(isActive) { this._active = isActive; }
    get active() { return this._active; }
    /**
     * Indicates whether this `MapElement` receives mouse and touch events.
     * Defaults to `true`.
     */
    set clickable(isActive) { this._clickable = isActive; }
    get clickable() { return this._clickable; }
    /**
     * The unique identifier for this `MapElement`.
     */
    set id(newId) { this._id = newId; }
    get id() { return this._id; }
    /**
     * The name of this `MapElement`.
     */
    set name(newName) { this._name = newName; }
    get name() { return this._name; }
    /**
     * The scale of this `MapElement`.
     */
    set scale(newScale) { this._scale = newScale; }
    get scale() { return this._scale; }
    /**
     * Indicates if this `MapElement` is currently visible. Defaults to `true`.
     */
    set visible(isVisible) { this._visible = isVisible; }
    get visible() { return this._visible; }
    /**
     * The z-index used to draw this `MapElement`.  Defaults to 1.
     */
    set zIndex(isVisible) { this._zIndex = isVisible; }
    get zIndex() { return this._zIndex; }
}

/**
 * Predefined map Symbols.
 */
var MarkerSymbolPaths;
(function (MarkerSymbolPaths) {
    MarkerSymbolPaths["computer"] = "M 8 36 c -2 0 -4 -2 -4 -4 v -20 c 0 -2 2 -4 4 -4 h 32 c 2 0 4 2 4 4 v 20 c 0 2 -2 4 -4 4 h 8 v 4 h -48 v -4 z M 8 12 h 32 v 20 h -32 z";
})(MarkerSymbolPaths || (MarkerSymbolPaths = {}));
/**
 * A styled vector path displayed as the icon of a `MapMarker`.
 */
class MarkerSymbol {
    constructor() {
        this._alt = false;
        this._anchor = new Coordinate(0, 0);
        this._path = '';
        /**
         * The name of the path.  This is appended to the class added to the path and
         * allows for custom styling based on each type of path.
         */
        this._pathName = '';
        this._rotation = 0;
        this._size = { width: 1, height: 1 };
    }
    /**
     * A flag indicating if this symbol should use an alternate style.
     */
    set alt(a) { this._alt = a; }
    get alt() { return this._alt; }
    /**
     * The position of the symbol relative to the marker.
     */
    set anchor(c) { this._anchor = c; }
    get anchor() { return this._anchor; }
    /**
     * The symbol's path specified using a built-in string or a custom path
     * expressed using SVG path notation.
     */
    set path(p) {
        // If supplied path is a string matching the name of a key of MarkerSymbolPaths,
        // set the path to that, otherwise just p. No check if string is valid SVG
        // path.
        if (p in MarkerSymbolPaths) {
            this._pathName = p;
            this._path = MarkerSymbolPaths[p];
        }
        else {
            this._pathName = 'custom';
            this._path = p;
        }
    }
    get path() { return this._path; }
    /**
     * The angle by which to rotate the symbol, expressed clockwise in degrees.
     */
    set rotation(r) { this._rotation = r % 360; } // Clamp to [0,360]
    get rotation() { return this._rotation; }
    /**
     * The size of the symbol
     */
    set size(s) { this._size = s; }
    get size() { return this._size; }
    /**
     * Renders the DOM necessary to display this symbol.
     */
    render() {
        const sx = this._size.width / 48;
        const sy = this._size.height / 48;
        const tr = this._anchor.toPathString();
        const ro = this._rotation;
        const t = `translate(${tr}) rotate(${ro}) scale(${sx}, ${sy})`;
        const pathClass = {
            'rl-svg__symbol': true,
        };
        pathClass[`rl-svg__symbol--${this._pathName}`] = true;
        pathClass[`rl-svg__symbol--${this._pathName}-alt`] = this._alt;
        return (h("path", { class: pathClass, transform: t, d: this._path }));
    }
}

const CONTROL_SIZE = 72;
/**
 * A type of `MapElement`.  A marker is an image or text rendered at a specific
 * point on a Map.
 */
class MapMarker extends MapElement {
    constructor(opts) {
        super(opts);
        this._anchor = new Coordinate(0, 0);
        this._available = false;
        this._label = '';
        this._opacity = 1;
        this._position = new Coordinate(0, 0);
        if (opts) {
            this.anchor = opts.anchor ? opts.anchor : new Coordinate(0, 0);
            this.available = opts.available ? opts.available : false;
            this.icon = opts.icon ? opts.icon : undefined;
            this.label = (opts.label !== undefined) ? opts.label : '';
            this.opacity = (opts.opacity !== undefined) ? opts.opacity : 1;
            this.position = opts.position;
        }
    }
    /**
     * An offset relative to the `position` of the marker.
     */
    set anchor(newAnchor) { this._anchor = newAnchor; }
    get anchor() { return this._anchor; }
    /**
     * A flag indicating if the this `MapMarker` represents an active or inactive
     * point.  When the marker is available, it gets a matching `--available` CSS
     * class added.
     */
    set available(isAvailable) { this._available = isAvailable; }
    get available() { return this._available; }
    /**
     * The icon that will be drawn for this `MapMarker`.
     */
    set icon(newIcon) { this._icon = newIcon; }
    get icon() { return this._icon; }
    /**
     * A line of text dispayed on the map either on its own as a simple text
     * label or along side the `icon`.
     */
    set label(newLabel) { this._label = newLabel; }
    get label() { return this._label; }
    /**
     * The opacity of the marker, 1 being fully opaque and 0 being fully
     * transparent.
     */
    set opacity(newOpacity) { this._opacity = Math.max(0, Math.min(1, newOpacity)); }
    get opacity() { return this._opacity; }
    /**
     * The overall position of the marker on the `Map`.
     */
    set position(newPosition) { this._position = newPosition; }
    get position() { return this._position; }
    /**
     * Moves this marker by a given amount.
     * @param delta The amount to move by
     */
    move(delta) {
        this._position.translate(delta);
    }
    render() {
        if (!this.visible || this._id === -1 &&
            this._position.equals(new Coordinate(0, 0))) {
            return undefined;
        }
        const rectClass = {
            'rl-svg__rect': true,
            'rl-svg__rect--activated': this.active,
        };
        const contents = [];
        const rect = (h("rect", { class: rectClass, rx: "12", ry: "12", height: CONTROL_SIZE, width: CONTROL_SIZE }));
        if (this._icon === undefined) {
            if (this._label !== '') {
                contents.push((h("text", { y: CONTROL_SIZE - 11, x: CONTROL_SIZE / 2 }, this._label)));
            }
        }
        else {
            this._icon.size = { width: CONTROL_SIZE, height: CONTROL_SIZE };
            if (this._icon instanceof MarkerSymbol) {
                this._icon.alt = this._available;
            }
            contents.push(this._icon.render());
        }
        contents.push(rect);
        const gTrans = `translate(${this.position.toPathString()})`;
        const gClass = {
            'rl-svg__marker': true,
            'rl-svg__marker--clickable': this._clickable,
        };
        return (h("g", { id: `${this._id}`, class: gClass, tabIndex: this._clickable ? 0 : undefined, transform: gTrans }, contents));
    }
}

/**
 * A type of `MapElement`.  A polygon is a shape drawn onto a `Map` based on a
 * path consisting of three or more `Coordinates`.
 */
class MapPolygon extends MapElement {
    constructor() {
        super(...arguments);
        this._open = false;
        /**
         * A string in SVG Path notation used to draw this `MapPolygon`.
         */
        this._path = '';
        this._points = [];
    }
    /**
     * If set to true, this `MapPolygon` is not automatically closed.
     */
    set open(open) {
        this._open = open;
        this._path = pathFromCoordinateArray(this._points, open);
    }
    get open() { return this._open; }
    /**
     * An array of `Coordinate`s that correspond to the points of this `MapPolygon`.
     */
    set points(newPoint) {
        this._points = newPoint;
        this._path = pathFromCoordinateArray(newPoint, this._open);
    }
    get points() { return this._points; }
    /**
     * Add a new point to the array of points that represent this `MapRegion`.
     * @param c The `Coordinate` representing the point to add
     * @param index The index where the point should be added.  If not specified,
     * adds the point at the end
     */
    addPoint(c, index) {
        const pts = this._points;
        this.points = index !== undefined ?
            [...pts.slice(0, index), c, ...pts.slice(index)] :
            [...pts, c];
    }
    /**
     * Move this `MapRegion` by a specified offset.
     * @param delta The amount to move by
     */
    move(delta) {
        const p = this.points;
        p.forEach(c => c.translate(delta));
        this.points = [...p];
    }
    /**
     * Move a single point of this `MapRegion` by a specified amount.
     * @param delta The amount to move by
     * @param index The index of the point to move
     */
    movePoint(delta, index) {
        const p = this.points;
        p[index].translate(delta);
        this.points = [...p];
        // this.points = arrayReplace(p, index, p[index].translate(delta));
    }
    /**
     * Remove a point from the array of points that represent this `MapRegion`.
     * If no index is specified, the last point is removed.
     * @param index The index of the point to remove
     */
    removePoint(index) {
        const p = this._points;
        if (index !== undefined) {
            p.splice(index, 1);
        }
        else {
            p.splice(p.length - 1, 1);
        }
        this.points = [...p];
    }
    /**
     * Renders the `MapPolygon`.
     */
    render() {
        if (this._visible === false) {
            return undefined;
        }
        const classname = {
            'rl-svg__polygon': true,
            'rl-svg__polygon--activated': this._active && !this._open,
            'rl-svg__polygon--open': this._open,
        };
        return (h("g", { id: `${this._id}`, class: classname },
            h("path", { d: this._path, tabIndex: 0, "aria-label": this._name, role: "button" })));
    }
    /**
     * Generates the DOM necessary to display the control points for this
     * `MapElement`.  There are control points at the median and endpoint of each
     * edge.  This allows any point of the `MapElement` to be moved as well as new
     * point created on each line.
     */
    renderControls() {
        // if (!this._active && !this._open) {
        //   return undefined;
        // }
        const pts = this._points;
        // If there are no points, or element is not visible, don't render controls.
        if (this._visible === false || pts.length === 0) {
            return undefined;
        }
        // Create a control for each point.  Don't create a control for the last
        // point of an open path though.
        const points = pts.filter((_, idx) => (pts.length === 1 || (this._open && idx === pts.length - 1 ? false : true))).map((i, index) => {
            const controls = [(h("circle", { cx: i.x, cy: i.y, r: 8 / this._scale, "data-index": index * 2, class: "rl-svg__control" }))];
            // Create a midpoint control.  This allows an edge segment to be split.
            if (!this._open) {
                const mid = (index === pts.length - 1) ?
                    Coordinate.midpoint(i, pts[0]) :
                    Coordinate.midpoint(i, pts[index + 1]);
                controls.push((h("circle", { cx: mid.x, cy: mid.y, r: 8 / this._scale, "data-index": index * 2 + 1, class: "rl-svg__midpoint" })));
            }
            return controls;
        });
        // Render an additional line to 'connect the dots' of the control points.
        const path = (h("path", { class: "rl-svg__polygon-outline", d: pathFromCoordinateArray(pts, this._open) }));
        return [path, points];
    }
}

/**
 * An image displayed as the icon of a `MapMarker`.
 */
class MarkerIcon {
    constructor() {
        this._anchor = new Coordinate(0, 0);
        this._originalSize = { width: 1, height: 1 };
        this._size = { width: 1, height: 1 };
        this._scale = 1;
        this._url = '';
    }
    /**
     * The position of the icon relative to the marker.
     */
    set anchor(c) { this._anchor = c; }
    get anchor() { return this._anchor; }
    /**
     * The natural size of the image without any scaling.
     */
    set originalSize(s) { this._originalSize = s; }
    get originalSize() { return this._originalSize; }
    /**
     * The size of the image after scaling.
     */
    set size(s) { this._size = s; }
    get size() { return this._size; }
    /**
     * The factor by which to scale the image.
     */
    set scale(s) { this._scale = s; }
    get scale() { return this._scale; }
    /**
     * The URL of the image.
     */
    set url(url) { this._url = url; }
    get url() { return this._url; }
    /**
     * Renders the DOM necessary to display the icon.
     */
    render() {
        const t = `translate(${this._anchor.toPathString()}) scale(${this._scale})`;
        return (h("image", { class: "rl-svg__icon", 
            // Don't set width.  Some icons are not square and leaving width
            // unset will set the width automatically while keeping aspect.
            height: this.size.height, transform: t, xlinkHref: this._url }));
    }
}

const HYSTERESIS = 8;
// Default maximum scale (zoom) factor.
const DEFAULT_MAX_SCALE = 3;
// Default minimum scale (zoom) factor.
const DEFAULT_MIN_SCALE = 1;
// The different states the Map/MapEditor can be in.
const STATES = {
    NORMAL: 0,
    GESTURE_DOWN: 1,
    DRAGGING: 2,
    ZOOMING: 3,
    ADD_REGION_INIT: 10,
    ADD_REGION_FIRST: 11,
    ADD_REGION: 12,
    ADD_POINT: 20,
    DRAG_ELEMENT: 30,
};
function computeLimits(iSize, sSize, scale) {
    return new DOMRect(Math.min(0, sSize.width - iSize.width * scale), // left / x
    Math.min(0, sSize.height - iSize.height * scale), // top / y
    Math.abs(iSize.width * scale - sSize.width), // width
    Math.abs(iSize.height * scale - sSize.height) // height
    );
}
/**
 * Gets the ID of a given Element within the Map SVG.  This makes use of the
 * known DOM structure of the SVG.
 * @param el The Element to use to find the ID.
 */
function getTargetId(el) {
    if (el instanceof SVGImageElement || el instanceof SVGRectElement ||
        el instanceof SVGPathElement || el instanceof SVGTextElement) {
        // The image (symbol), bounding rectangle, text or path were clicked.
        // In these cases, the parent (group) tag holds the ID.
        el = el.parentElement;
    }
    if (el instanceof SVGGElement) {
        return Number(el.id);
    }
    return undefined;
}
/**
 * Processes an array of `MapElement`s, turning them into an array of
 * `ParsedMapElement` objects.  These parsed map elements are then rendered onto
 * a map.
 * @param elements An array of `MapElement`s to process
 * @param scale The scale applied to the `ParsedElement`
 */
function parseElements(elements, scale = 1) {
    return Object.values(elements).map((el) => {
        const decoded = decodeCoordinates(el.points, true);
        let icon;
        if (typeof el.icon === 'string') {
            icon = new MarkerIcon();
            icon.url = el.icon;
        }
        else if (el.symbol !== undefined && el.symbol in MarkerSymbolPaths) {
            icon = new MarkerSymbol();
            icon.path = el.symbol;
        }
        let newEl;
        if (decoded.points instanceof Coordinate) {
            newEl = new MapMarker();
            newEl.icon = icon;
            newEl.position = decoded.points;
            newEl.label = (typeof el.icon !== 'string' && typeof el.symbol !== 'string') ? el.name : '';
            newEl.available = el.available || false;
        }
        else {
            newEl = new MapPolygon();
            newEl.points = decoded.points;
        }
        newEl.id = el.id;
        newEl.name = el.name;
        newEl.visible = el.enabled;
        newEl.scale = scale;
        newEl.clickable = el.clickable !== undefined ? el.clickable : true;
        return newEl;
    });
}

export { STATES as a, Coordinate as b, DEFAULT_MAX_SCALE as c, DEFAULT_MIN_SCALE as d, parseElements as e, coordinateFromEvent as f, getTargetId as g, HYSTERESIS as h, computeLimits as i, MapPolygon as j, MapMarker as k, encodeCoordinates as l };
