import { d as registerInstance, e as createEvent, f as h, g as getElement, h as Host } from './rlwc-ca835c31.js';
import { a as STATES, b as Coordinate, c as DEFAULT_MAX_SCALE, d as DEFAULT_MIN_SCALE, e as parseElements, f as coordinateFromEvent, g as getTargetId, h as HYSTERESIS, i as computeLimits } from './chunk-dd9ee2fa.js';

class RLMap {
    constructor(hostRef) {
        registerInstance(this, hostRef);
        /**
         * The array of MapElements currently being displayed.  Created from the
         * `elements` Prop with additional internal information added.
         */
        this.processedElements = [];
        /**
         * The original scale factor used to size the SVG so it fits into the SVG and
         * covers the entire SVG.
         */
        this.initialScale = 1;
        /**
         * The current state of the Map.
         */
        this.state = STATES.NORMAL;
        /**
         * Flag indicating if the map image has loaded or not.
         */
        this.imageLoaded = false;
        /**
         * The current size of the SVG element used to display the map.
         */
        this.size = { width: 1, height: 1 };
        /**
         * The factor by which the Map contents are changed to fit within the SVG.
         */
        this.svgScale = 1;
        /**
         * The x,y translation applied to move the map contents within the bounds of
         * the SVG.
         */
        this.svgTransform = new Coordinate(0, 0);
        /**
         *  The maximum scale factor.
         */
        this.maxScale = DEFAULT_MAX_SCALE;
        /**
         * The minimum scale factor.
         */
        this.minScale = DEFAULT_MIN_SCALE;
        this.elementDeselected = createEvent(this, "elementDeselected", 7);
        this.elementSelected = createEvent(this, "elementSelected", 7);
    }
    onMapImageChanged() {
        if (this.mapImage === undefined) {
            return;
        }
        const img = new Image();
        img.src = this.mapImage;
        this.imageLoaded = false;
        img.onload = () => {
            this.imageLoaded = true;
            if (this.imgSize &&
                this.imgSize.width === img.width &&
                this.imgSize.height === img.height) {
                return;
            }
            this.imgSize = { width: img.width, height: img.height };
            this.computeScale();
            this.limits = this.computeLimits();
        };
    }
    /**
     * Handle when the list of specified elements changes.
     */
    onElementsChanged() {
        // if (this.elements !== undefined) {
        this.processedElements = parseElements(this.elements);
        // }
    }
    componentDidLoad() {
        this.onMapImageChanged();
        this.onElementsChanged();
        this.onResize();
    }
    /**
     * Handles when the user triggers a touch/down event to start interaction with
     * the map.
     * @param e The triggering event.
     */
    onGestureDown(e) {
        this.start = this.last = coordinateFromEvent(e, this.root);
        if (this.state === STATES.NORMAL) {
            this.state = STATES.GESTURE_DOWN;
        }
        // Get the ID of the event target, if it is the correct type.
        const id = getTargetId(e.target);
        if (id !== undefined) {
            this.targetElement = this.processedElements.find(i => i.id === id);
        }
    }
    /**
     * Handles when the user triggers a movement event.
     * @param e The triggering event.
     */
    onGestureMove(e) {
        if (this.state === STATES.NORMAL) {
            return;
        }
        // Convert the coordinate to SVG coordinate space.
        const point = coordinateFromEvent(e, this.root);
        switch (this.state) {
            case STATES.GESTURE_DOWN:
                const dist = Coordinate.squareDistance(this.start, point);
                if (dist > HYSTERESIS) {
                    this.state = STATES.DRAGGING;
                }
                break;
            case STATES.DRAGGING:
                if (this.limits !== undefined) {
                    const delta = Coordinate.difference(point, this.last);
                    this.svgTransform = Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                }
                break;
            default:
        }
        this.last = point;
    }
    /**
     * Handles when the user ends their interaction with the Map.
     * @param e The triggering event.
     */
    onGestureUp(e) {
        const point = coordinateFromEvent(e, this.root);
        switch (this.state) {
            case STATES.DRAGGING:
                if (this.limits !== undefined) {
                    const delta = Coordinate.difference(point, this.last);
                    this.svgTransform = Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                }
                break;
            case STATES.GESTURE_DOWN:
                if (this.targetElement) {
                    if (this.targetElement !== this.activeElement && this.targetElement.clickable) {
                        this._setActiveElement(this.targetElement);
                        this.targetElement = undefined;
                    }
                }
                else {
                    if (this.activeElement) {
                        this.elementDeselected.emit(this.activeElement);
                        this._clearActiveElement();
                    }
                }
                break;
            default:
        }
        this.state = STATES.NORMAL;
    }
    /**
     * Handles when the user leaves the bounds of the Map entirely.
     * @param e The triggering event.
     */
    onMouseLeave(e) {
        if (this.state === STATES.DRAGGING ||
            this.state === STATES.GESTURE_DOWN) {
            this.state = STATES.NORMAL;
            const point = coordinateFromEvent(e, this.root);
            if (this.state === STATES.DRAGGING && this.limits !== undefined) {
                const delta = Coordinate.difference(point, this.last);
                this.svgTransform = Coordinate.sum(this.svgTransform, delta).limit(this.limits).round();
                this.state = STATES.NORMAL;
            }
        }
    }
    /**
     * Handles when the user performs a resize action.
     */
    onResize() {
        clearTimeout(this.resizeDebounce);
        this.resizeDebounce = setTimeout(_ => {
            // Use a 150ms delay to debounce multiple resize calls and effectively
            // perform the resize action only when the user is 'done' resizing.
            this.computeSvgSize();
            this.computeScale();
            this.limits = this.computeLimits();
            this.svgTransform = this.svgTransform.clone().limit(this.limits);
        }, 150);
    }
    /**
     * Handles when the user performs a mouse wheen event.  The Map will be zoomed
     * in or out depending on the direction of the wheel.
     *
     * @param e The triggering event.
     */
    onWheel(e) {
        const oldPos = this.toSvgSpace(new Coordinate(e.clientX, e.clientY));
        if (e.deltaY < 0) {
            if (this.svgScale * 1.05263157895 < this.initialScale * this.maxScale) {
                this.svgScale *= 1.05263157895;
            }
        }
        else {
            if (this.svgScale * 0.95 > this.initialScale * this.minScale) {
                this.svgScale *= 0.95;
            }
        }
        this.limits = this.computeLimits();
        const newPos = this.toSvgSpace(new Coordinate(e.clientX, e.clientY));
        const delta = Coordinate.difference(newPos, oldPos).scale(this.svgScale);
        this.svgTransform.translate(delta).limit(this.limits).round();
    }
    onEnter(e) {
        if (e.key === 'enter' && e.target && e.target instanceof SVGElement &&
            (e.target.classList.contains('rl-map-polygon') ||
                e.target.classList.contains('rl-map-marker'))) {
            const id = Number(e.target.id);
            const el = this.processedElements.find(i => i.id === id);
            this._setActiveElement(el);
        }
    }
    /**
     * Clears the currently active element.
     */
    async clearActiveElement() {
        this._clearActiveElement();
    }
    /**
     * Sets the element with the specified ID to active.
     *
     * @param id The ID of the element to set as active.
     */
    async setActiveElement(id) {
        if (this.processedElements.length > 0) {
            this._setActiveElement(this.processedElements.find(i => i.id === id), false);
        }
    }
    /**
     * Clears the currently active element.
     */
    _clearActiveElement() {
        if (this.activeElement) {
            this.activeElement.active = false;
        }
        this.activeElement = undefined;
    }
    /**
     * Updates the limits used to restrict the how far the SVG transform can be
     * moved keeping the map from being panning beyond the edge of the image.
     */
    computeLimits() {
        const { imgSize, size } = this;
        if (imgSize !== undefined) {
            return computeLimits(imgSize, size, this.svgScale);
        }
        else {
            return new DOMRect(0, 0, 1, 1);
        }
    }
    /**
     * Computes the scale factor needed to resize the full map image to be as
     * large as possible, and cropping it vertically/horizontally so no empty
     * space exists if the container has different aspect from the image. This is
     * like CSS `background-size: cover` or SVG `aspectRatio xMidYMid slice`.
     */
    computeScale() {
        const { imgSize, size } = this;
        if (imgSize === undefined) {
            this.svgScale = this.initialScale = 1;
        }
        else {
            this.svgScale = this.initialScale =
                Math.max(size.width / imgSize.width, size.height / imgSize.height);
        }
    }
    /**
     * Determines the current size of the SVG element serving as the container
     * for the entire map.
     */
    computeSvgSize() {
        const root = this.root.querySelector('svg');
        const rect = root && root.getBoundingClientRect();
        if (rect !== null) {
            this.size = { width: rect.width, height: rect.height };
        }
    }
    /**
     * Sets the currently active element of the Map.
     *
     * @param el The Element that will be set as the active element.
     */
    _setActiveElement(el, shouldEmit = true) {
        if (!el || (this.activeElement && el.id === this.activeElement.id)) {
            return;
        }
        this.clearActiveElement();
        this.activeElement = el;
        this.activeElement.active = true;
        // Emit an event with the original (un-parsed) element as the new 'active'
        // element.  Don't emit an event if the flag is set.
        if (shouldEmit) {
            this.elementSelected.emit(this.elements[this.activeElement.id]);
        }
    }
    /**
     * Scales a given `Coordinate` by the inverse of the current `svgScale`.
     * Returns a new `Coordinate` with its values scaled and rounded.
     * @param c A `Coordinate` scale.
     */
    toSvgScale(c) {
        return c.clone().scale(1 / this.svgScale).round();
    }
    /**
     * Converts a given `Coordinate` object in screen coordinate space to SVG
     * coordinate space.
     * @param c A `Coordinate` to convert to SVG coordinate space.
     */
    toSvgSpace(c) {
        return Coordinate.difference(this.toSvgScale(c), this.toSvgScale(this.svgTransform));
    }
    hostData() {
        return {
            class: {
                'rl-map': true,
                'rl-map--loaded': this.imageLoaded,
            },
        };
    }
    __stencil_render() {
        const m = [
            this.svgScale, 0, 0, this.svgScale,
            this.svgTransform.x,
            this.svgTransform.y,
        ];
        const matrix = 'matrix(' + m.join(',') + ')';
        return (h("svg", { class: "rl-svg" }, h("g", { transform: matrix }, h("image", { xlinkHref: this.mapImage !== undefined ? this.mapImage : undefined }), this.processedElements.map(el => el.render()))));
    }
    get root() { return getElement(this); }
    static get watchers() { return {
        "mapImage": ["onMapImageChanged"],
        "elements": ["onElementsChanged"]
    }; }
    render() { return h(Host, this.hostData(), this.__stencil_render()); }
    static get style() { return "\@-webkit-keyframes fade-in{0%{opacity:0}to{opacity:1}}\@keyframes fade-in{0%{opacity:0}to{opacity:1}}.rl-map{display:inline-block;width:100%;height:100%;opacity:0;overflow:hidden}.rl-map--loaded{opacity:1;-webkit-animation:fade-in .18s ease-in;animation:fade-in .18s ease-in}.rl-svg{width:100%;height:100%}.rl-svg__marker--clickable:hover rect,.rl-svg__marker:focus rect{outline:none;fill:var(--rl-svg__focus-fill,rgba(255,255,83,.3))}.rl-svg__marker--clickable:hover rect{cursor:pointer}.rl-svg__marker text{font-size:var(--rl-svg__text-size,48px);-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.rl-svg__polygon{fill:var(--rl-svg__polygon-fill,rgba(0,76,155,.6));stroke:var(--rl-svg__polygon-fill,#004c9b);stroke-width:4}.rl-svg__polygon--activated,.rl-svg__polygon:focus{outline:none;fill:var(--rl-svg__focus-fill,rgba(255,255,83,.3))}.rl-svg__polygon--activated{stroke:var(--rl-svg__active-stroke,#c7ab00);stroke-width:8}.rl-svg__polygon:hover{outline:none;fill:var(--rl-svg__focus-fill,rgba(255,255,83,.3));cursor:pointer}.rl-svg__polygon-outline{fill:none;stroke:var(--rl-map-element-active-stroke,#c7ab00);stroke-width:2}.rl-svg__polygon path{outline:none}.rl-svg__rect{fill:none;stroke:none}.rl-svg__rect--activated{outline:none;stroke:var(--rl-svg__active-stroke,#c7ab00);stroke-width:8}.rl-svg__symbol{fill:var(--rl-svg__symbol-fill,rgba(0,76,155,.6));stroke:var(--rl-svg__symbol-fill,#004c9b);stroke-width:1;fill-rule:evenodd}.rl-svg__symbol--alt{fill:var(--rl-svg__symbol-fill-alt,rgba(255,255,83,.5));stroke:var(--rl-svg__symbol-fill-alt,#c7ab00)}"; }
}

export { RLMap as rl_map };
