/**
 * Class for representing coordinates and positions.
 * Adapted from Google Closure Library.
 */
export declare class Coordinate {
    /**
     * The X-value
     */
    x: number;
    /**
     * The Y-value
     */
    y: number;
    /**
     * Create a new Coordinate.
     */
    constructor(x: number, y: number);
    /**
     * Returns a new copy of the coordinate.
     */
    clone(): Coordinate;
    /**
     * Returns a nice string representation the coordinate.
     */
    toString(): string;
    /**
     * Returns a basic string representation of the Coordinate used to easily
     * create SVG path strings.
     */
    toPathString(): string;
    /**
     * Returns whether the specified value is equal to this coordinate.
     * @param other Some other value.
     */
    equals(other: Coordinate): boolean;
    /**
     * Rounds the x and y fields to the next larger integer values.
     */
    ceil(): Coordinate;
    /**
     * Rounds the x and y fields to the next smaller integer values.
     */
    floor(): Coordinate;
    /**
     * Translates this coordinate by the given offsets. If a `Coordinate`
     * is given, then the x and y values are translated by the coordinate's x
     * and y. Otherwise, x and y are translated by `tx` and `ty` respectively.
     *
     * @param tx The value to translate x by or the
     *     the coordinate to translate this coordinate by.
     * @param ty The value to translate y by.
     */
    translate(tx: number | Coordinate, ty?: number): Coordinate;
    /**
     * Scales this coordinate by the given scale factors. The x and y values are
     * scaled by `sx` and `sy` respectively.  If `sy` is not given, then `sx` is
     * used for both x and y.
     *
     * @param sx The scale factor to use for the x dimension.
     * @param opt_sy The scale factor to use for the y dimension.
     */
    scale(sx: number, sy?: number): Coordinate;
    /**
     * Rotates this coordinate clockwise about the origin (or, optionally, the
     * given center) by the given angle, in radians.
     *
     * @param radians The angle by which to rotate this coordinate
     *     clockwise about the given center, in radians.
     * @param center The center of rotation. Defaults
     *     to (0, 0) if not given.
     */
    rotateRadians(radians: number, center?: Coordinate): this;
    /**
     * Rotates this coordinate clockwise about the origin (or, optionally, the
     * given center) by the given angle, in degrees.
     *
     * @param degrees The angle by which to rotate this coordinate
     *     clockwise about the given center, in degrees.
     * @param opt_center The center of rotation. Defaults
     *     to (0, 0) if not given.
     */
    rotateDegrees(degrees: number, center?: Coordinate): this;
    /**
     * Restricts each parameter of the Coordinate such that
     * `left <= x <= left + width` and `top <= y <= top + height`.
     *
     * @param limits A rectangle to restrict the Coordinate to.
     */
    limit(limits: DOMRect): this;
    /**
     * Round the values of the `Coordinate` to a given precision (number of
     * decimals).  Defaults to a precision of 0, or rounding to the nearest
     * integer.
     *
     * @param precision The number of decimals to round to
     */
    round(precision?: number): this;
    /**
     * Compares two `Coordinate`s for equlaity.
     * @param a The first `Coordinate`.
     * @param b The second `Coordinate`.
     */
    static equals(a: Coordinate, b: Coordinate): boolean;
    /**
     * Returns the distance between two `Coordinate`s.
     * @param a The first `Coordinate`.
     * @param b The second `Coordinate`.
     */
    static distance(a: Coordinate, b: Coordinate): number;
    /**
     * Returns the magnitude (distance from origin) of a `Coordinate`.
     * @param a The first `Coordinate`.
     */
    static magnitude(a: Coordinate): number;
    /**
     * Returns the squared distance between two `Coordinate`s. Squared distances
     * can be used for comparisons when the actual value is not required.
     *
     * Performance note: eliminating the square root is an optimization often used
     * in lower-level languages, but the speed difference is not nearly as
     * pronounced in JavaScript (only a few percent.)
     *
     * @param a The first `Coordinate`.
     * @param b The second `Coordinate`.
     */
    static squareDistance(a: Coordinate, b: Coordinate): number;
    /**
     * Returns the difference between two `Coordinate`s.
     * @param a The first `Coordinate`.
     * @param b The second `Coordinate`.
     */
    static difference(a: Coordinate, b?: Coordinate): Coordinate;
    /**
     * Returns the sum of two coordinates as a new `Coordinate`.
     * @param a The first `Coordinate`.
     * @param b The second `Coordinate`.
     */
    static sum(a: Coordinate, b: Coordinate): Coordinate;
    /**
     * Returns the midpoint of two coordinates as a new `Coordinate`.
     * @param a The first `Coordinate`.
     * @param b The second `Coordinate`.
     */
    static midpoint(a: Coordinate, b: Coordinate): Coordinate;
}
